@@ -9,20 +9,30 @@
 //
 
 #import "VLCDropboxTableViewController.h"
+#import "VLCHTTPUploaderController.h"
+#import "GHRevealViewController.h"
+#import "VLCMenuTableViewController.h"
+#import "VLCDownloadViewController.h"
 
 @class VLCPlaylistViewController;
 @class PAPasscodeViewController;
 @interface VLCAppDelegate : UIResponder <UIApplicationDelegate>
 
 - (void)updateMediaList;
+- (void)disableIdleTimer;
+- (void)activateIdleTimer;
 
 @property (nonatomic, readonly) VLCPlaylistViewController *playlistViewController;
 @property (nonatomic, readonly) VLCDropboxTableViewController *dropboxTableViewController;
+@property (nonatomic, readonly) VLCDownloadViewController *downloadViewController;
 
 @property (nonatomic, strong) UIWindow *window;
 
-@property (nonatomic, strong) UINavigationController *navigationController;
+@property (nonatomic, strong) GHRevealViewController *revealController;
+@property (nonatomic, strong) VLCMenuTableViewController *menuViewController;
 
 @property (nonatomic, retain) NSDate *nextPasscodeCheckDate;
 
+@property (nonatomic) VLCHTTPUploaderController *uploadController;
+
 @end
