@@ -40,6 +40,7 @@
     BOOL _displayRemainingTime;
     BOOL _positionSet;
     BOOL _playerIsSetup;
+    BOOL _isScrubbing;
 }
 
 @property (nonatomic, strong) UIPopoverController *masterPopoverController;
@@ -93,11 +94,46 @@
     self.videoFilterView.hidden = YES;
     _videoFiltersHidden = YES;
     _hueLabel.text = NSLocalizedString(@"VFILTER_HUE", @"");
+    _hueSlider.accessibilityLabel = _hueLabel.text;
+    _hueSlider.isAccessibilityElement = YES;
     _contrastLabel.text = NSLocalizedString(@"VFILTER_CONTRAST", @"");
+    _contrastSlider.accessibilityLabel = _contrastLabel.text;
+    _contrastSlider.isAccessibilityElement = YES;
     _brightnessLabel.text = NSLocalizedString(@"VFILTER_BRIGHTNESS", @"");
+    _brightnessSlider.accessibilityLabel = _brightnessLabel.text;
+    _brightnessSlider.isAccessibilityElement = YES;
     _saturationLabel.text = NSLocalizedString(@"VFILTER_SATURATION", @"");
+    _saturationSlider.accessibilityLabel = _saturationLabel.text;
+    _saturationSlider.isAccessibilityElement = YES;
     _gammaLabel.text = NSLocalizedString(@"VFILTER_GAMMA", @"");
+    _gammaSlider.accessibilityLabel = _gammaLabel.text;
+    _gammaSlider.isAccessibilityElement = YES;
     _playbackSpeedLabel.text = NSLocalizedString(@"PLAYBACK_SPEED", @"");
+    _playbackSpeedSlider.accessibilityLabel = _playbackSpeedLabel.text;
+    _playbackSpeedSlider.isAccessibilityElement = YES;
+
+    _positionSlider.accessibilityLabel = NSLocalizedString(@"PLAYBACK_POSITION", @"");
+    _positionSlider.isAccessibilityElement = YES;
+    _timeDisplay.isAccessibilityElement = YES;
+
+    _audioSwitcherButton.accessibilityLabel = NSLocalizedString(@"CHOOSE_AUDIO_TRACK", @"");
+    _audioSwitcherButton.isAccessibilityElement = YES;
+    _subtitleSwitcherButton.accessibilityLabel = NSLocalizedString(@"CHOOSE_SUBTITLE_TRACK", @"");
+    _subtitleSwitcherButton.isAccessibilityElement = YES;
+    _playbackSpeedButton.accessibilityLabel = _playbackSpeedLabel.text;
+    _playbackSpeedButton.isAccessibilityElement = YES;
+    _videoFilterButton.accessibilityLabel = NSLocalizedString(@"VIDEO_FILTER", @"");
+    _videoFilterButton.isAccessibilityElement = YES;
+    _resetVideoFilterButton.accessibilityLabel = NSLocalizedString(@"VIDEO_FILTER_RESET_BUTTON", @"");
+    _resetVideoFilterButton.isAccessibilityElement = YES;
+    _aspectRatioButton.accessibilityLabel = NSLocalizedString(@"VIDEO_ASPECT_RATIO_BUTTON", @"");
+    _aspectRatioButton.isAccessibilityElement = YES;
+    _playPauseButton.accessibilityLabel = NSLocalizedString(@"PLAY_PAUSE_BUTTON", @"");
+    _playPauseButton.isAccessibilityElement = YES;
+    _bwdButton.accessibilityLabel = NSLocalizedString(@"BWD_BUTTON", @"");
+    _bwdButton.isAccessibilityElement = YES;
+    _fwdButton.accessibilityLabel = NSLocalizedString(@"FWD_BUTTON", @"");
+    _fwdButton.isAccessibilityElement = YES;
 
     _scrubHelpLabel.text = NSLocalizedString(@"PLAYBACK_SCRUB_HELP", @"");
 
@@ -121,6 +157,8 @@
     if ([self hasExternalDisplay])
         [self showOnExternalDisplay];
 
+    self.trackNameLabel.text = self.artistNameLabel.text = self.albumNameLabel.text = @"";
+
     _movieView.userInteractionEnabled = NO;
     UITapGestureRecognizer *tapOnVideoRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(toggleControlsVisible)];
     tapOnVideoRecognizer.delegate = self;
@@ -156,9 +194,22 @@
     [self.aspectRatioButton setBackgroundImage:[UIImage imageNamed:@"ratioButton"] forState:UIControlStateNormal];
     [self.aspectRatioButton setBackgroundImage:[UIImage imageNamed:@"ratioButtonHighlight"] forState:UIControlStateHighlighted];
     [self.aspectRatioButton setImage:[UIImage imageNamed:@"ratioIcon"] forState:UIControlStateNormal];
-    [self.toolbar setBackgroundImage:[UIImage imageNamed:@"seekbarBg"] forBarMetrics:UIBarMetricsDefault];
-    [self.backButton setBackgroundImage:[UIImage imageNamed:@"playbackDoneButton"] forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
-    [self.backButton setBackgroundImage:[UIImage imageNamed:@"playbackDoneButtonHighlight"] forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
+    if (SYSTEM_RUNS_IN_THE_FUTURE) {
+        self.backButton.tintColor = [UIColor colorWithRed:(190.0f/255.0f) green:(190.0f/255.0f) blue:(190.0f/255.0f) alpha:1.];
+        self.toolbar.tintColor = [UIColor whiteColor];
+        self.toolbar.barTintColor = [UIColor colorWithWhite:0.f alpha:1.f];
+
+        CGRect rect = self.positionSlider.frame;
+        rect.origin.y = rect.origin.y - 5.;
+        self.positionSlider.frame = rect;
+        rect = self.resetVideoFilterButton.frame;
+        rect.origin.y = rect.origin.y + 5.;
+        self.resetVideoFilterButton.frame = rect;
+    } else {
+        [self.toolbar setBackgroundImage:[UIImage imageNamed:@"seekbarBg"] forBarMetrics:UIBarMetricsDefault];
+        [self.backButton setBackgroundImage:[UIImage imageNamed:@"playbackDoneButton"] forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
+        [self.backButton setBackgroundImage:[UIImage imageNamed:@"playbackDoneButtonHighlight"] forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
+    }
 
     /* this looks a bit weird, but we need to support iOS 5 and should show the same appearance */
     UISlider *volumeSlider = nil;
@@ -215,8 +266,10 @@
 
     [self.navigationController setNavigationBarHidden:YES animated:YES];
 
-    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
-        [UIApplication sharedApplication].statusBarStyle = UIStatusBarStyleBlackTranslucent;
+    if (!SYSTEM_RUNS_IN_THE_FUTURE) {
+        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
+            [UIApplication sharedApplication].statusBarStyle = UIStatusBarStyleBlackTranslucent;
+    }
 
     [self _startPlayback];
 
@@ -229,7 +282,9 @@
     if (_playerIsSetup)
         return;
 
-    _mediaPlayer = [[VLCMediaPlayer alloc] init];
+    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+
+    _mediaPlayer = [[VLCMediaPlayer alloc] initWithOptions:@[[NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFont, [defaults objectForKey:kVLCSettingSubtitlesFont]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFontColor, [defaults objectForKey:kVLCSettingSubtitlesFontColor]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFontSize, [defaults objectForKey:kVLCSettingSubtitlesFontSize]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingDeinterlace, [defaults objectForKey:kVLCSettingDeinterlace]]]];
     [_mediaPlayer setDelegate:self];
     [_mediaPlayer setDrawable:self.movieView];
 
@@ -243,12 +298,18 @@
         self.title = [self.mediaItem title];
         media = [VLCMedia mediaWithURL:[NSURL URLWithString:self.mediaItem.url]];
         self.mediaItem.unread = @(NO);
+
+        if (self.mediaItem.isAlbumTrack) {
+            self.trackNameLabel.text = self.mediaItem.albumTrack.title;
+            self.artistNameLabel.text = self.mediaItem.albumTrack.artist;
+            self.albumNameLabel.text = self.mediaItem.albumTrack.album.name;
+        } else
+            self.trackNameLabel.text = self.artistNameLabel.text = self.albumNameLabel.text = @"";
     } else {
         media = [VLCMedia mediaWithURL:self.url];
         self.title = @"Network Stream";
     }
 
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
     [media addOptions:
      @{kVLCSettingStretchAudio :
      [[defaults objectForKey:kVLCSettingStretchAudio] boolValue] ? kVLCSettingStretchAudioOnValue : kVLCSettingStretchAudioOffValue, kVLCSettingTextEncoding : [defaults objectForKey:kVLCSettingTextEncoding], kVLCSettingSkipLoopFilter : [defaults objectForKey:kVLCSettingSkipLoopFilter]}];
@@ -298,6 +359,7 @@
 
     self.positionSlider.value = 0.;
     [self.timeDisplay setTitle:@"" forState:UIControlStateNormal];
+    self.timeDisplay.accessibilityLabel = @"";
 
     if (![self _isMediaSuitableForDevice]) {
         UIAlertView * alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"DEVICE_TOOSLOW_TITLE", @"") message:[NSString stringWithFormat:NSLocalizedString(@"DEVICE_TOOSLOW", @""), [[UIDevice currentDevice] model], self.mediaItem.title] delegate:self cancelButtonTitle:NSLocalizedString(@"BUTTON_CANCEL", @"") otherButtonTitles:NSLocalizedString(@"BUTTON_OPEN", @""), nil];
@@ -382,7 +444,8 @@
         _idleTimer = nil;
     }
     [self.navigationController setNavigationBarHidden:NO animated:YES];
-    [UIApplication sharedApplication].statusBarStyle = UIStatusBarStyleBlackOpaque;
+    if (!SYSTEM_RUNS_IN_THE_FUTURE)
+        [UIApplication sharedApplication].statusBarStyle = UIStatusBarStyleBlackOpaque;
     [[UIApplication sharedApplication] setStatusBarHidden:NO withAnimation:UIStatusBarAnimationFade];
     [super viewWillDisappear:animated];
 
@@ -440,13 +503,6 @@
     return YES;
 }
 
-
-- (void)motionEnded:(UIEventSubtype)motion withEvent:(UIEvent *)event
-{
-    if (motion == UIEventSubtypeMotionShake)
-        [[VLCBugreporter sharedInstance] handleBugreportRequest];
-}
-
 - (void)remoteControlReceivedWithEvent:(UIEvent *)event
 {
     switch (event.subtype) {
@@ -585,6 +641,7 @@
     [self performSelector:@selector(_setPositionForReal) withObject:nil afterDelay:0.3];
     VLCTime *newPosition = [VLCTime timeWithInt:(int)(_positionSlider.value * self.mediaItem.duration.intValue)];
     [self.timeDisplay setTitle:newPosition.stringValue forState:UIControlStateNormal];
+    self.timeDisplay.accessibilityLabel = [NSString stringWithFormat:@"%@: %@", NSLocalizedString(@"PLAYBACK_POSITION", @""), newPosition.stringValue];
     _positionSet = NO;
     [self _resetIdleTimer];
 }
@@ -601,11 +658,13 @@
 {
     [self _updateScrubLabel];
     self.scrubIndicatorView.hidden = NO;
+    _isScrubbing = YES;
 }
 
 - (IBAction)positionSliderTouchUp:(id)sender
 {
     self.scrubIndicatorView.hidden = YES;
+    _isScrubbing = NO;
 }
 
 - (void)_updateScrubLabel
@@ -634,7 +693,10 @@
 }
 
 - (void)mediaPlayerTimeChanged:(NSNotification *)aNotification {
-    self.positionSlider.value = [_mediaPlayer position];
+    if (!_isScrubbing) {
+        self.positionSlider.value = [_mediaPlayer position];
+    }
+
     if (_displayRemainingTime)
         [self.timeDisplay setTitle:[[_mediaPlayer remainingTime] stringValue] forState:UIControlStateNormal];
     else
@@ -912,11 +974,16 @@
 {
     [self _saveCurrentState];
 
+    _mediaPlayer.currentVideoTrackIndex = 0;
+
     if (![[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingContinueAudioInBackgroundKey] boolValue]) {
-        [_mediaPlayer pause];
-        _shouldResumePlaying = YES;
-    } else
-        _mediaPlayer.currentVideoTrackIndex = 0;
+        if ([_mediaPlayer isPlaying]) {
+            [_mediaPlayer pause];
+            _shouldResumePlaying = YES;
+        }
+    }
+
+    glFinish();
 }
 
 - (void)applicationDidEnterBackground:(NSNotification *)notification
@@ -926,11 +993,12 @@
 
 - (void)applicationDidBecomeActive:(NSNotification *)notification
 {
+    _mediaPlayer.currentVideoTrackIndex = 1;
+
     if (_shouldResumePlaying) {
         _shouldResumePlaying = NO;
         [_mediaPlayer play];
-    } else
-        _mediaPlayer.currentVideoTrackIndex = 1;
+    }
 }
 
 - (void)_updateExportedPlaybackInformation
@@ -943,10 +1011,17 @@
 
     /* we omit artwork for now since we had to read it from storage as we can't access
      * the artwork cache at the moment - FIXME? */
-    NSDictionary *currentlyPlayingTrackInfo = @{ MPMediaItemPropertyTitle : currentFile.title,
-                                                 MPMediaItemPropertyPlaybackDuration : @(currentFile.duration.intValue / 1000.),
-                                                 MPNowPlayingInfoPropertyElapsedPlaybackTime : @(_mediaPlayer.time.intValue / 1000.),
-                                                 MPNowPlayingInfoPropertyPlaybackRate : @(_mediaPlayer.rate) };
+    NSMutableDictionary *currentlyPlayingTrackInfo = [NSMutableDictionary dictionaryWithObjectsAndKeys: currentFile.title, MPMediaItemPropertyTitle, @(currentFile.duration.intValue / 1000.), MPMediaItemPropertyPlaybackDuration, @(_mediaPlayer.time.intValue / 1000.), MPNowPlayingInfoPropertyElapsedPlaybackTime, @(_mediaPlayer.rate), MPNowPlayingInfoPropertyPlaybackRate, nil];
+    if ([currentFile isAlbumTrack]) {
+        MLAlbumTrack *track = currentFile.albumTrack;
+        if (track.artist.length > 0)
+            [currentlyPlayingTrackInfo setObject:track.artist forKey:MPMediaItemPropertyArtist];
+        if (track.title.length > 0)
+            [currentlyPlayingTrackInfo setObject:track.title forKey:MPMediaItemPropertyTitle];
+        if (track.album.name.length > 0)
+            [currentlyPlayingTrackInfo setObject:track.album.name forKey:MPMediaItemPropertyAlbumTitle];
+        [currentlyPlayingTrackInfo setObject:[NSNumber numberWithInt:[track.trackNumber intValue]] forKey:MPMediaItemPropertyAlbumTrackNumber];
+    }
 
     [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = currentlyPlayingTrackInfo;
 }
