@@ -12,6 +12,8 @@
 #import "VLCAppDelegate.h"
 #import "VLCPlaylistViewController.h"
 #import "UIBarButtonItem+Theme.h"
+#import "UINavigationController+Theme.h"
+#import "VLCMenuTableViewController.h"
 
 @interface VLCOpenNetworkStreamViewController ()
 {
@@ -36,13 +38,14 @@
     [self.openButton setTitle:NSLocalizedString(@"BUTTON_OPEN", @"") forState:UIControlStateNormal];
     [self.privateModeLabel setText:NSLocalizedString(@"PRIVATE_PLAYBACK_TOGGLE", @"")];
     self.title = NSLocalizedString(@"OPEN_NETWORK", @"");
-
-    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
-        self.navigationItem.leftBarButtonItem = [UIBarButtonItem themedBackButtonWithTarget:self andSelector:@selector(goBack:)];
+    self.navigationItem.leftBarButtonItem = [UIBarButtonItem themedRevealMenuButtonWithTarget:self andSelector:@selector(goBack:)];
+    [self.whatToOpenHelpLabel setText:NSLocalizedString(@"OPEN_NETWORK_HELP", @"")];
 }
 
 - (void)viewWillAppear:(BOOL)animated
 {
+    self.navigationController.navigationBar.translucent = NO;
+
     if ([[UIPasteboard generalPasteboard] containsPasteboardTypes:@[@"public.url", @"public.text"]]) {
         NSURL *pasteURL = [[UIPasteboard generalPasteboard] valueForPasteboardType:@"public.url"];
         if (!pasteURL || [[pasteURL absoluteString] isEqualToString:@""]) {
@@ -63,6 +66,9 @@
 
 - (void)viewWillDisappear:(BOOL)animated
 {
+    if (SYSTEM_RUNS_IN_THE_FUTURE)
+        self.navigationController.navigationBar.translucent = YES;
+
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
     [defaults setObject:[NSArray arrayWithArray:_recentURLs] forKey:kVLCRecentURLs];
     [defaults setBool:self.privateToggleSwitch.on forKey:kVLCPrivateWebStreaming];
@@ -85,16 +91,20 @@
 
 - (IBAction)goBack:(id)sender
 {
-    [self.navigationController popViewControllerAnimated:YES];
+    [[(VLCAppDelegate*)[UIApplication sharedApplication].delegate revealController] toggleSidebar:![(VLCAppDelegate*)[UIApplication sharedApplication].delegate revealController].sidebarShowing duration:kGHRevealSidebarDefaultAnimationDuration];
 }
 
 - (IBAction)openButtonAction:(id)sender
 {
     if ([self.urlField.text length] > 0) {
         if (!self.privateToggleSwitch.on) {
+            if ([_recentURLs indexOfObject:self.urlField.text] != NSNotFound)
+                [_recentURLs removeObject:self.urlField.text];
+
             if (_recentURLs.count >= 15)
                 [_recentURLs removeLastObject];
             [_recentURLs addObject:self.urlField.text];
+
             [self.historyTableView reloadData];
         }
         [self _openURLStringAndDismiss:self.urlField.text];
@@ -160,12 +170,8 @@
 - (void)_openURLStringAndDismiss:(NSString *)url
 {
     VLCAppDelegate* appDelegate = [UIApplication sharedApplication].delegate;
-    [appDelegate.playlistViewController openMovieFromURL:[NSURL URLWithString:url]];
-
-    [self dismissModalViewControllerAnimated:YES];
-
-    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
-        [appDelegate.playlistViewController.addMediaPopoverController dismissPopoverAnimated:YES];
+    [appDelegate.menuViewController selectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] animated:NO scrollPosition:UITableViewScrollPositionNone];
+    [appDelegate.playlistViewController performSelector:@selector(openMovieFromURL:) withObject:[NSURL URLWithString:url] afterDelay:kGHRevealSidebarDefaultAnimationDuration];
 }
 
 @end
