@@ -20,6 +20,9 @@
 @property (nonatomic, strong) IBOutlet VLCLinearProgressIndicator *progressView;
 @property (nonatomic, strong) IBOutlet UIButton *removeMediaButton;
 @property (nonatomic, strong) IBOutlet UIImageView *mediaIsUnreadView;
+@property (nonatomic, strong) IBOutlet UILabel *seriesNameLabel;
+@property (nonatomic, strong) IBOutlet UILabel *artistNameLabel;
+@property (nonatomic, strong) IBOutlet UILabel *albumNameLabel;
 
 // Temporary workaround: until better solution
 @property (nonatomic, weak) AQGridView *gridView;
