@@ -11,8 +11,13 @@
 #import <UIKit/UIKit.h>
 #import "AQGridView.h"
 
+#define EXPERIMENTAL_LIBRARY 1
+
+#define kVLCLibraryModeAllFiles 0
+#define kVLCLibraryModeAllAlbums 1
+#define kVLCLibraryModeAllSeries 2
+
 @class VLCMovieViewController;
-@class VLCMenuViewController;
 @class EmptyLibraryView;
 
 @interface VLCPlaylistViewController : UIViewController <UITabBarDelegate, UIPopoverControllerDelegate>
@@ -26,18 +31,20 @@
 
 @property (nonatomic, strong) EmptyLibraryView *emptyLibraryView;
 
-
 - (IBAction)leftButtonAction:(id)sender;
 
 - (void)updateViewContents;
 - (void)openMovieFromURL:(NSURL *)url;
 - (void)removeMediaObject:(MLFile *)mediaObject;
 
+- (void)setLibraryMode:(NSUInteger)mode;
+
 @end
 
 @interface EmptyLibraryView: UIView
 
 @property (nonatomic, strong) IBOutlet UILabel *emptyLibraryLabel;
 @property (nonatomic, strong) IBOutlet UILabel *emptyLibraryLongDescriptionLabel;
+@property (nonatomic, strong) IBOutlet UIActivityIndicatorView *activityIndicator;
 
 @end
