@@ -12,17 +12,24 @@
 #import "VLCMovieViewController.h"
 #import "VLCPlaylistTableViewCell.h"
 #import "VLCPlaylistGridView.h"
-#import "VLCMenuViewController.h"
 #import "UINavigationController+Theme.h"
 #import "NSString+SupportedMedia.h"
 #import "VLCBugreporter.h"
+#import "VLCAppDelegate.h"
+#import "UIBarButtonItem+Theme.h"
+
+
+#ifndef UIStatusBarStyleLightContent
+#define UIStatusBarStyleLightContent 1
+#endif
 
 @implementation EmptyLibraryView
 @end
 
-@interface VLCPlaylistViewController () <AQGridViewDataSource, AQGridViewDelegate,
-                                         UITableViewDataSource, UITableViewDelegate> {
+@interface VLCPlaylistViewController () <AQGridViewDataSource, AQGridViewDelegate, UITableViewDataSource, UITableViewDelegate, MLMediaLibrary> {
     NSMutableArray *_foundMedia;
+    NSUInteger _libraryMode;
+    UIBarButtonItem *_menuButton;
 }
 @end
 
@@ -43,6 +50,8 @@
         self.view = _gridView;
     }
 
+    _libraryMode = kVLCLibraryModeAllFiles;
+
     self.view.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
     self.emptyLibraryView = [[[NSBundle mainBundle] loadNibNamed:@"VLCEmptyLibraryView" owner:self options:nil] lastObject];
 }
@@ -53,9 +62,7 @@
 {
     [super viewDidLoad];
 
-    UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"menuCone"] style:UIBarButtonItemStyleBordered target:self action:@selector(leftButtonAction:)];
-    [addButton setBackgroundImage:[UIImage imageNamed:@"button"] forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
-    [addButton setBackgroundImage:[UIImage imageNamed:@"buttonHighlight"] forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
+    _menuButton = [UIBarButtonItem themedRevealMenuButtonWithTarget:self andSelector:@selector(leftButtonAction:)];
 
     /* After day 354 of the year, the usual VLC cone is replaced by another cone
      * wearing a Father Xmas hat.
@@ -66,14 +73,18 @@
     [[NSCalendar alloc] initWithCalendarIdentifier:NSGregorianCalendar];
     NSUInteger dayOfYear = [gregorian ordinalityOfUnit:NSDayCalendarUnit inUnit:NSYearCalendarUnit forDate:[NSDate date]];
     if (dayOfYear >= 354)
-        addButton.image = [UIImage imageNamed:@"vlc-xmas"];
+        _menuButton.image = [UIImage imageNamed:@"vlc-xmas"];
 
-    self.navigationItem.leftBarButtonItem = addButton;
+    self.navigationItem.leftBarButtonItem = _menuButton;
 
-    [self.editButtonItem setBackgroundImage:[UIImage imageNamed:@"button"]
-                                   forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
-    [self.editButtonItem setBackgroundImage:[UIImage imageNamed:@"buttonHighlight"]
-                                   forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
+    if (SYSTEM_RUNS_IN_THE_FUTURE)
+        self.editButtonItem.tintColor = [UIColor whiteColor];
+    else {
+        [self.editButtonItem setBackgroundImage:[UIImage imageNamed:@"button"]
+                                       forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
+        [self.editButtonItem setBackgroundImage:[UIImage imageNamed:@"buttonHighlight"]
+                                       forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
+    }
 
     if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
         _gridView.separatorStyle = AQGridViewCellSeparatorStyleEmptySpace;
@@ -83,7 +94,7 @@
         _tableView.rowHeight = [VLCPlaylistTableViewCell heightOfCell];
         _tableView.separatorColor = [UIColor colorWithWhite:.122 alpha:1.];
     }
-    self.navigationItem.titleView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"title"]];
+    self.title = NSLocalizedString(@"LIBRARY_ALL_FILES", @"");
 
     _emptyLibraryView.emptyLibraryLabel.text = NSLocalizedString(@"EMPTY_LIBRARY", @"");
     _emptyLibraryView.emptyLibraryLongDescriptionLabel.lineBreakMode = UILineBreakModeWordWrap;
@@ -100,10 +111,29 @@
 
 - (void)viewDidAppear:(BOOL)animated
 {
-    [self performSelector:@selector(updateViewContents) withObject:nil afterDelay:.0];
+    [super viewDidAppear:animated];
+
+    if ([[MLMediaLibrary sharedMediaLibrary] libraryNeedsUpgrade]) {
+        self.navigationItem.rightBarButtonItem = nil;
+        self.navigationItem.leftBarButtonItem = nil;
+        self.emptyLibraryView.emptyLibraryLabel.text = NSLocalizedString(@"UPGRADING_LIBRARY", @"");
+        self.emptyLibraryView.emptyLibraryLongDescriptionLabel.hidden = YES;
+        [self.emptyLibraryView.activityIndicator startAnimating];
+        self.emptyLibraryView.frame = self.view.frame;
+        self.title = @"";
+        [self.view addSubview:self.emptyLibraryView];
+
+        [[MLMediaLibrary sharedMediaLibrary] setDelegate: self];
+        [[MLMediaLibrary sharedMediaLibrary] performSelectorInBackground:@selector(upgradeLibrary) withObject:nil];
+        return;
+    }
+
+    if (_foundMedia.count < 1)
+        [self updateViewContents];
     [[MLMediaLibrary sharedMediaLibrary] performSelector:@selector(libraryDidAppear) withObject:nil afterDelay:1.];
 
-    [super viewDidAppear:animated];
+    if (SYSTEM_RUNS_IN_THE_FUTURE)
+        [UIApplication sharedApplication].statusBarStyle = UIStatusBarStyleLightContent;
 }
 
 - (void)viewDidDisappear:(BOOL)animated
@@ -148,16 +178,17 @@
 
 - (void)_displayEmptyLibraryViewIfNeeded
 {
-    if (_foundMedia.count > 0) {
-        if (self.emptyLibraryView.superview)
-            [self.emptyLibraryView removeFromSuperview];
+    if (self.emptyLibraryView.superview)
+        [self.emptyLibraryView removeFromSuperview];
 
-        self.navigationItem.rightBarButtonItem = self.editButtonItem;
-    } else {
+    if (_foundMedia.count <= 0) {
         self.emptyLibraryView.frame = self.view.frame;
         [self.view addSubview:self.emptyLibraryView];
-        self.navigationItem.rightBarButtonItem = nil;
     }
+    if (_libraryMode == kVLCLibraryModeAllFiles && _foundMedia.count > 0)
+        self.navigationItem.rightBarButtonItem = self.editButtonItem;
+    else
+        self.navigationItem.rightBarButtonItem = nil;
 
     if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone) {
         _tableView.separatorStyle = (_foundMedia.count > 0)? UITableViewCellSeparatorStyleSingleLine:
@@ -165,21 +196,68 @@
     }
 }
 
+- (void)libraryUpgradeComplete
+{
+    self.emptyLibraryView.emptyLibraryLongDescriptionLabel.hidden = NO;
+    self.emptyLibraryView.emptyLibraryLabel.text = NSLocalizedString(@"EMPTY_LIBRARY", @"");
+    [self.emptyLibraryView.activityIndicator stopAnimating];
+    self.title = NSLocalizedString(@"LIBRARY_ALL_FILES", @"");
+    self.navigationItem.leftBarButtonItem = _menuButton;
+    [self.emptyLibraryView removeFromSuperview];
+
+    [self updateViewContents];
+}
+
+- (void)libraryWasUpdated
+{
+    NSLog(@"libraryWasUpdated");
+    [self updateViewContents];
+}
+
 - (void)updateViewContents
 {
-    _foundMedia = [NSMutableArray arrayWithArray:[MLFile allFiles]];
+    if (_libraryMode == kVLCLibraryModeAllAlbums) {
+        NSArray *rawAlbums = [MLAlbum allAlbums];
+        _foundMedia = [[NSMutableArray alloc] init];
+        NSUInteger count = rawAlbums.count;
+        MLAlbum *album;
 
+        for (NSUInteger x = 0; x < count; x++) {
+            album = rawAlbums[x];
+            if (album.name.length > 0 && album.tracks.count > 0)
+                [_foundMedia addObject:album];
+        }
+        rawAlbums = nil;
+    } else if (_libraryMode == kVLCLibraryModeAllSeries) {
+        NSArray *rawShows = [MLShow allShows];
+        _foundMedia = [[NSMutableArray alloc] init];
+        NSUInteger count = rawShows.count;
+        MLShow *show;
+
+        for (NSUInteger x = 0; x < count; x++) {
+            show = rawShows[x];
+            if (show.name.length > 0 && show.episodes.count > 0)
+                [_foundMedia addObject:show];
+        }
+        rawShows = nil;
+    } else
+        _foundMedia = [NSMutableArray arrayWithArray:[MLFile allFiles]];
+
+    [self reloadViews];
+}
+
+- (void)reloadViews
+{
     if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone)
         [self.tableView reloadData];
     else {
         [self.gridView reloadData];
-        [self.gridView setNeedsLayout];
+        [self.gridView performSelector:@selector(reloadData) withObject:nil afterDelay:2.];
     }
 
     [self _displayEmptyLibraryViewIfNeeded];
 }
 
-
 #pragma mark - Table View
 
 - (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
@@ -225,12 +303,34 @@
 
 - (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
 {
-    MLFile *mediaObject = _foundMedia[indexPath.row];
-    if (!self.movieViewController)
-        self.movieViewController = [[VLCMovieViewController alloc] initWithNibName:nil bundle:nil];
+    [tableView deselectRowAtIndexPath:indexPath animated:YES];
 
-    self.movieViewController.mediaItem = mediaObject;
-    [self.navigationController pushViewController:self.movieViewController animated:YES];
+    NSManagedObject *currentObject = _foundMedia[indexPath.row];
+    if ([currentObject isKindOfClass:[MLAlbum class]]) {
+        _foundMedia = [NSMutableArray arrayWithArray:[[(MLAlbum *)currentObject tracks] allObjects]];
+        self.navigationItem.leftBarButtonItem = [UIBarButtonItem themedBackButtonWithTarget:self andSelector:@selector(backToAllItems:)];
+        [self.navigationItem.leftBarButtonItem setTitle:NSLocalizedString(@"LIBRARY_MUSIC", @"")];
+        self.title = [(MLAlbum*)currentObject name];
+        [self reloadViews];
+    } else if ([currentObject isKindOfClass:[MLShow class]]) {
+        _foundMedia = [NSMutableArray arrayWithArray:[[(MLShow *)currentObject episodes] allObjects]];
+        self.navigationItem.leftBarButtonItem = [UIBarButtonItem themedBackButtonWithTarget:self andSelector:@selector(backToAllItems:)];
+        [self.navigationItem.leftBarButtonItem setTitle:NSLocalizedString(@"LIBRARY_SERIES", @"")];
+        self.title = [(MLShow*)currentObject name];
+        [self reloadViews];
+    } else {
+        if (!self.movieViewController)
+            self.movieViewController = [[VLCMovieViewController alloc] initWithNibName:nil bundle:nil];
+
+        if ([currentObject isKindOfClass:[MLFile class]])
+            self.movieViewController.mediaItem = (MLFile *)currentObject;
+        else if ([currentObject isKindOfClass:[MLAlbumTrack class]])
+            self.movieViewController.mediaItem = [(MLAlbumTrack*)currentObject files].anyObject;
+        else if ([currentObject isKindOfClass:[MLShowEpisode class]])
+            self.movieViewController.mediaItem = [(MLShowEpisode*)currentObject files].anyObject;
+
+        [self.navigationController pushViewController:self.movieViewController animated:YES];
+    }
 }
 
 #pragma mark - AQGridView
@@ -264,12 +364,32 @@
 {
     [self.gridView deselectItemAtIndex:index animated:YES];
 
-    MLFile *mediaObject = _foundMedia[index];
-    if (!self.movieViewController)
-        self.movieViewController = [[VLCMovieViewController alloc] initWithNibName:nil bundle:nil];
+    NSManagedObject *currentObject = _foundMedia[index];
+    if ([currentObject isKindOfClass:[MLAlbum class]]) {
+        _foundMedia = [NSMutableArray arrayWithArray:[[(MLAlbum *)currentObject tracks] allObjects]];
+        self.navigationItem.leftBarButtonItem = [UIBarButtonItem themedBackButtonWithTarget:self andSelector:@selector(backToAllItems:)];
+        [self.navigationItem.leftBarButtonItem setTitle:NSLocalizedString(@"LIBRARY_MUSIC", @"")];
+        self.title = [(MLAlbum*)currentObject name];
+        [self reloadViews];
+    } else if ([currentObject isKindOfClass:[MLShow class]]) {
+        _foundMedia = [NSMutableArray arrayWithArray:[[(MLShow *)currentObject episodes] allObjects]];
+        self.navigationItem.leftBarButtonItem = [UIBarButtonItem themedBackButtonWithTarget:self andSelector:@selector(backToAllItems:)];
+        [self.navigationItem.leftBarButtonItem setTitle:NSLocalizedString(@"LIBRARY_SERIES", @"")];
+        self.title = [(MLShow*)currentObject name];
+        [self reloadViews];
+    } else {
+        if (!self.movieViewController)
+            self.movieViewController = [[VLCMovieViewController alloc] initWithNibName:nil bundle:nil];
 
-    self.movieViewController.mediaItem = mediaObject;
-    [self.navigationController pushViewController:self.movieViewController animated:YES];
+        if ([currentObject isKindOfClass:[MLFile class]])
+            self.movieViewController.mediaItem = (MLFile *)currentObject;
+        else if ([currentObject isKindOfClass:[MLAlbumTrack class]])
+            self.movieViewController.mediaItem = [(MLAlbumTrack*)currentObject files].anyObject;
+        else if ([currentObject isKindOfClass:[MLShowEpisode class]])
+            self.movieViewController.mediaItem = [(MLShowEpisode*)currentObject files].anyObject;
+
+        [self.navigationController pushViewController:self.movieViewController animated:YES];
+    }
 }
 
 - (void)gridView:(AQGridView *)aGridView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndex:(NSUInteger)index
@@ -281,16 +401,23 @@
 #pragma mark - UI implementation
 - (void)setEditing:(BOOL)editing animated:(BOOL)animated
 {
+    if (_libraryMode != kVLCLibraryModeAllFiles)
+        return;
+
     [super setEditing:editing animated:animated];
 
     UIBarButtonItem *editButton = self.editButtonItem;
     NSString *editImage = editing? @"doneButton": @"button";
     NSString *editImageHighlight = editing? @"doneButtonHighlight": @"buttonHighlight";
-    [editButton setBackgroundImage:[UIImage imageNamed:editImage] forState:UIControlStateNormal
-                                 barMetrics:UIBarMetricsDefault];
-    [editButton setBackgroundImage:[UIImage imageNamed:editImageHighlight]
-                                   forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
-    [editButton setTitleTextAttributes: editing ? @{UITextAttributeTextShadowColor : [UIColor whiteColor], UITextAttributeTextColor : [UIColor blackColor]} : @{UITextAttributeTextShadowColor : [UIColor colorWithWhite:0. alpha:.37], UITextAttributeTextColor : [UIColor whiteColor]} forState:UIControlStateNormal];
+    if (SYSTEM_RUNS_IN_THE_FUTURE)
+        editButton.tintColor = [UIColor whiteColor];
+    else {
+        [editButton setBackgroundImage:[UIImage imageNamed:editImage] forState:UIControlStateNormal
+                                     barMetrics:UIBarMetricsDefault];
+        [editButton setBackgroundImage:[UIImage imageNamed:editImageHighlight]
+                                       forState:UIControlStateHighlighted barMetrics:UIBarMetricsDefault];
+        [editButton setTitleTextAttributes: editing ? @{UITextAttributeTextShadowColor : [UIColor whiteColor], UITextAttributeTextColor : [UIColor blackColor]} : @{UITextAttributeTextShadowColor : [UIColor colorWithWhite:0. alpha:.37], UITextAttributeTextColor : [UIColor whiteColor]} forState:UIControlStateNormal];
+    }
 
     if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
         [self.gridView setEditing:editing];
@@ -298,32 +425,23 @@
         [self.tableView setEditing:editing animated:YES];
 }
 
+- (UITableViewCellEditingStyle)tableView:(UITableView *)aTableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
+{
+    if (_libraryMode != kVLCLibraryModeAllFiles)
+        return UITableViewCellEditingStyleNone;
+
+    return UITableViewCellEditingStyleDelete;
+}
+
 - (IBAction)leftButtonAction:(id)sender
 {
-    if (self.menuViewController == nil) {
-        VLCMenuViewController *menuViewController = [[VLCMenuViewController alloc] initWithNibName:nil bundle:nil];
-        UINavigationController *navigationController = [[UINavigationController alloc] initWithRootViewController:menuViewController];
-        [navigationController loadTheme];
-        self.menuViewController = navigationController;
-        if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
-            navigationController.navigationBarHidden = YES;
-    }
+    [[(VLCAppDelegate*)[UIApplication sharedApplication].delegate revealController] toggleSidebar:![(VLCAppDelegate*)[UIApplication sharedApplication].delegate revealController].sidebarShowing duration:kGHRevealSidebarDefaultAnimationDuration];
+}
 
-    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
-        self.menuViewController.contentSizeForViewInPopover = self.menuViewController.view.frame.size;
-        if (self.addMediaPopoverController == nil) {
-            self.addMediaPopoverController = [[UIPopoverController alloc] initWithContentViewController:self.menuViewController];
-            self.addMediaPopoverController.delegate = self;
-        }
-
-        if (self.addMediaPopoverController.popoverVisible)
-            [self.addMediaPopoverController dismissPopoverAnimated:YES];
-        else
-            [self.addMediaPopoverController presentPopoverFromBarButtonItem:self.navigationItem.leftBarButtonItem
-                                                   permittedArrowDirections:UIPopoverArrowDirectionUp
-                                                                   animated:YES];
-    } else
-        [self.navigationController presentViewController:self.menuViewController animated:YES completion:NULL];
+- (IBAction)backToAllItems:(id)sender
+{
+    self.navigationItem.leftBarButtonItem = _menuButton;
+    [self setLibraryMode:_libraryMode];
 }
 
 /* deprecated in iOS 6 */
@@ -364,4 +482,18 @@
     self.movieViewController.url = url;
 }
 
+- (void)setLibraryMode:(NSUInteger)mode
+{
+    _libraryMode = mode;
+
+    if (_libraryMode == kVLCLibraryModeAllAlbums)
+        self.title = NSLocalizedString(@"LIBRARY_MUSIC", @"");
+    else if( _libraryMode == kVLCLibraryModeAllSeries)
+        self.title = NSLocalizedString(@"LIBRARY_SERIES", @"");
+    else
+        self.title = NSLocalizedString(@"LIBRARY_ALL_FILES", @"");
+
+    [self updateViewContents];
+}
+
 @end
