@@ -9,9 +9,9 @@ SDK=iphoneos6.1
 SDK_MIN=5.1
 VERBOSE=no
 CONFIGURATION="Release"
-TESTEDHASH=9419612ae
-TESTEDVLCKITHASH=790e7e31b
-TESTEDMEDIALIBRARYKITHASH=91f69d1f5
+TESTEDHASH=0d987c72d
+TESTEDVLCKITHASH=353bbbe1b
+TESTEDMEDIALIBRARYKITHASH=79fdd5760
 
 usage()
 {
@@ -143,6 +143,28 @@ fi
 if ! [ -e OBSlider ]; then
 git clone git://github.com/ole/OBSlider.git
 fi
+if ! [ -e GHSidebarNav ]; then
+git clone git://github.com/gresrun/GHSidebarNav.git
+cd GHSidebarNav
+git am ../../patches/ghsidebarnav/*.patch
+if [ $? -ne 0 ]; then
+git am --abort
+info "Applying the patches failed, aborting git-am"
+exit 1
+fi
+cd ..
+fi
+if ! [ -e upnpx ]; then
+UPNPXVERSION=1.2.4
+curl -O http://upnpx.googlecode.com/files/upnpx-${UPNPXVERSION}.tar.gz
+tar xf upnpx-${UPNPXVERSION}.tar.gz
+mv upnpx-${UPNPXVERSION} upnpx
+cd upnpx && patch -p1 < ${aspen_root_dir}/patches/upnpx/duration-selector-failure.patch
+cd ..
+fi
+if ! [ -e WhiteRaccoon ]; then
+git clone git://github.com/fkuehne/WhiteRaccoon.git
+fi
 if ! [ -e AQGridView ]; then
 git clone git://github.com/AlanQuatermain/AQGridView.git
 cd AQGridView
@@ -194,6 +216,7 @@ else
 fi
 framework_build="${aspen_root_dir}/ImportedSources/VLCKit/${xcbuilddir}"
 mlkit_build="${aspen_root_dir}/ImportedSources/MediaLibraryKit/${xcbuilddir}"
+upnpx_build="${aspen_root_dir}/ImportedSources/upnpx/projects/xcode4/upnpx/${xcbuilddir}"
 
 spushd MediaLibraryKit
 rm -f External/MobileVLCKit
@@ -204,6 +227,7 @@ spopd #ImportedSources
 
 ln -sf ${framework_build} External/MobileVLCKit
 ln -sf ${mlkit_build} External/MediaLibraryKit
+ln -sf ${upnpx_build} External/upnpx
 
 #
 # Build time
@@ -237,6 +261,10 @@ spushd MediaLibraryKit
 buildxcodeproj MediaLibraryKit
 spopd
 
+spushd upnpx/projects/xcode4/upnpx
+buildxcodeproj upnpx
+spopd
+
 spopd # ImportedSources
 
 
